# -*- coding: utf-8 -*-

from urlparse import urlparse
from zope.interface import implements
from redturtle.video.interfaces import IVideoEmbedCode
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile

class VideoEmbedCode(object):
    """ VideoEmbedCode 
    provides a way to have a html code to embed video in a web page 

    >>> from zope.interface import implements
    >>> from redturtle.video.interfaces import IRTRemoteVideo
    >>> from redturtle.video.interfaces import IVideoEmbedCode
    >>> from zope.component import getMultiAdapter
    >>> from redturtle.video.tests.base import TestRequest

    >>> class RemoteVideo(object):
    ...     implements(IRTRemoteVideo)
    ...     remoteUrl = 'http://www.site.com/viedo.mpeg'
    ...     def getRemoteUrl(self):
    ...         return self.remoteUrl

    >>> remotevideo = RemoteVideo()
    >>> adapter = getMultiAdapter((remotevideo, TestRequest()), IVideoEmbedCode)
    >>> adapter.getVideoLink()
    'http://www.site.com/viedo.mpeg'

    """
    implements(IVideoEmbedCode)
    template = ViewPageTemplateFile('basevideoembedcode_template.pt')

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def getVideoLink(self):
        return self.context.getRemoteUrl()

    def getWidth(self):
        return self.context.getWidth()
    
    def getHeight(self):
        return self.context.getHeight()    

    def __call__(self):
        return self.template()




from zope.interface import Interface


class IVideoEmbedCode(Interface):
    """Marker interface to provide a video embed html code"""

    def getVideoLink():
        """Return a link to the remote video resource"""
    
    def getWidth():
        """Obtain video width in pixel"""

    def getHeight():
        """Obtain video height in pixel"""




"""Definition of the RTInternalVideo content type
"""

from zope.interface import implements

from AccessControl import ClassSecurityInfo
from Products.CMFCore import permissions
from Products.Archetypes import atapi
from Products.ATContentTypes.content import base
from Products.ATContentTypes.content import schemata
from Products.ATContentTypes.content.file import ATFileSchema
from Products.ATContentTypes.content.image import ATImageSchema
from Products.ATContentTypes.lib.imagetransform import ATCTImageTransform

from collective.flowplayer.interfaces import IFlowPlayable

from redturtle.video import videoMessageFactory as _
from redturtle.video.interfaces import IRTInternalVideo
from redturtle.video.config import PROJECTNAME
from redturtle.video.content.default import DefaultVideo
from redturtle.video.content.video_schema import VIDEO_SCHEMA

RTInternalVideoSchema = ATFileSchema.copy().copy() + VIDEO_SCHEMA.copy()

# Set storage on fields copied from ATContentTypeSchema, making sure
# they work well with the python bridge properties.

RTInternalVideoSchema['title'].storage = atapi.AnnotationStorage()
RTInternalVideoSchema['title'].required = True
RTInternalVideoSchema['description'].storage = atapi.AnnotationStorage()

imageField = ATImageSchema['image'].copy()
imageField.required = False
imageField.primary = False
imageField.widget.description = _(u'help_video_image',
                                  default=u'Can be used to provide splash image when needed')
imageField.validators = None

RTInternalVideoSchema.addField(imageField)
RTInternalVideoSchema.moveField('image', after='file')

fileField = RTInternalVideoSchema['file']
fileField.widget.label = _(u'label_video_file',
                          default=u'Video file')
fileField.widget.description = _(u'help_video_file',
                                default=u'Put there the video file in a compatible format')

schemata.finalizeATCTSchema(RTInternalVideoSchema, moveDiscussion=False)


class RTInternalVideo(base.ATCTContent, ATCTImageTransform, DefaultVideo):
    """A video file with screenshot"""
    implements(IRTInternalVideo, IFlowPlayable)

    meta_type = "RTInternalVideo"
    schema = RTInternalVideoSchema

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')

    # -*- Your ATSchema to Python Property Bridges Here ... -*-

    security = ClassSecurityInfo()

    security.declareProtected(permissions.View, 'tag')
    def tag(self, **kwargs):
        """Generate image tag using the api of the ImageField
        """
        return self.getField('image').tag(self, **kwargs)

    def __bobo_traverse__(self, REQUEST, name):
        """Transparent access to image scales
        """
        if name.startswith('image'):
            field = self.getField('image')
            image = None
            if name == 'image':
                image = field.getScale(self)
            else:
                scalename = name[len('image_'):]
                scalename.replace(".jpg", "")
                if scalename in field.getAvailableSizes(self):
                    image = field.getScale(self, scale=scalename)
            if image is not None and not isinstance(image, basestring):
                # image might be None or '' for empty images
                return image

        return base.ATCTContent.__bobo_traverse__(self, REQUEST, name)

    security.declareProtected(permissions.View, 'size')
    def size(self):
        """Get size video size
        """
        return self.get_size()

    security.declareProtected(permissions.View, 'get_size')
    def get_size(self):
        """ZMI / Plone get size method
        """
        f = self.getPrimaryField()
        if f is None:
            return 0
        return f.get_size(self) or 0

    def hasSplashScreenImage(self):
        """Boolean value to know if an image is available"""
        if self.getImage():
            return True
        return False

atapi.registerType(RTInternalVideo, PROJECTNAME)



"""Common configuration constants
"""

PROJECTNAME = 'redturtle.video'

ADD_PERMISSIONS = {
    # -*- extra stuff goes here -*-
    'RTRemoteVideo': 'redturtle.video: Add RTRemoteVideo',
    'RTInternalVideo': 'redturtle.video: Add RTInternalVideo',
}



# -*- coding: utf-8 -*-

from zope.event import notify

from Products.Archetypes.event import ObjectInitializedEvent

from collective.flowplayer.interfaces import IVideo
from collective.flowplayer.interfaces import IMediaInfo


def migrateRTInternalVideo(context):
    from plone.app.blob.migrations import migrate
    return migrate(context, 'RTInternalVideo')


# helper to build custom blob migrators for the given type
# some code stolen from the migration of plone.app.blob
def makeMigrator(context, meta_type):
    """ generate a migrator for the given at-based portal type """
    from Products.contentmigration.archetypes import InplaceATItemMigrator
    
    class FlowplayerMigrator(InplaceATItemMigrator):
        src_portal_type = 'File'
        src_meta_type = meta_type
        dst_meta_type = 'RTInternalVideo'
        dst_portal_type = 'RTInternalVideo'

        def last_migrate_mediaData(self):
            media = IMediaInfo(self.old)
            notify(ObjectInitializedEvent(self.new))

    return FlowplayerMigrator


def migrateFlowplayerToRedTurtleVideo(context):
    from Products.contentmigration.walker import CustomQueryWalker
    migrator = makeMigrator(context, 'ATFile')
    walker = CustomQueryWalker(context, migrator, use_savepoint=True,
                               query={'object_provides': IVideo.__identifier__})
    walker.go()
    output = walker.getOutput()
    migrator = makeMigrator(context, 'ATBlob')
    walker = CustomQueryWalker(context, migrator, use_savepoint=True,
                               query={'object_provides': IVideo.__identifier__})
    walker.go()
    output += walker.getOutput()
    return output.splitlines()




# -*- coding: utf-8 -*-

import cStringIO
from zope.component import getMultiAdapter
from redturtle.video.tests.base import TestCase

class TestView(TestCase):

    def afterSetUp(self):
        self.setRoles(('Contributor', ))
        portal = self.portal
        portal.invokeFactory(type_name="File", id="atfile")
        f = getattr(portal, 'atfile')
        f.edit(title="Simple ATFile", file=self.getVideoFile())


    def test_getEmbedCode_accessible(self):
        # BBB: FAKE TESTS right now
        # seems that during tests the allowed_interface limitation are not taken
        self.portal.invokeFactory(type_name='RTInternalVideo', id='internal-video')
        video = getattr(self.portal, 'internal-video')
        video.edit(title='Internal Video', file=self.getVideoFile())
        #view = getMultiAdapter((video, self.portal.REQUEST), name='flowplayer')
        view = self.portal['internal-video'].restrictedTraverse('@@flowplayer')
        self.assertTrue(view.getEmbedCode())



def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestView))
    return suite



# -*- coding: utf-8 -*-

import unittest
from redturtle.video.tests import base

from Products.CMFPlone.utils import getToolByName


class TestSetup(base.TestCase):

    def afterSetUp(self):
        self.qi = getToolByName(self.portal, 'portal_quickinstaller')

    def test_skin_installed(self):
        self.failUnless(self.qi.isProductInstalled('collective.flowplayer'))

    def test_catalogindexes(self):
        catalog_indexes = self.portal.portal_catalog.indexes()
        self.failUnless("hasSplashScreenImage" in catalog_indexes)
        self.failUnless("getYear" in catalog_indexes)
        self.failUnless("getDuration" in catalog_indexes)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestSetup))
    return suite



#from hachoir_parser.guess import createParser
#from hachoir_metadata.metadata import extractMetadata
#from hachoir_core.error import HachoirError
#from hachoir_core.stream import InputStreamError

# Includes above someway broke the stdout when used at module level

from redturtle.video import logger

def extract(filename):
    """Extract the metadata from the media file"""

    from hachoir_parser.guess import createParser
    from hachoir_metadata.metadata import extractMetadata
    from hachoir_core.error import HachoirError
    from hachoir_core.stream import InputStreamError

    filename = unicode(filename)

    try:
        parser = createParser(filename)
    except InputStreamError, err:
        logger.warning("Stream error! %s" % unicode(err))
        return None

    if not parser:
        logger.warning("Unable to create parser.")
        return None
    try:
        metadata = extractMetadata(parser)
    except HachoirError, err:
        logger.warning("Stream error! %s" % unicode(err))
        return None

    if metadata is None:
        logger.warning("unable to extract metadata.")
        return None

    return metadata



from zope.interface import implements
from zope.component import getMultiAdapter

from plone.portlets.interfaces import IPortletDataProvider
from plone.app.portlets.portlets import base

from zope import schema
from zope.formlib import form

from plone.memoize.instance import memoize

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.app.vocabularies.catalog import SearchableTextSourceBinder
from plone.app.form.widgets.uberselectionwidget import UberSelectionWidget

from Products.ATContentTypes.interface import IATTopic, IATFolder

from redturtle.video.interfaces import IRTVideo
from redturtle.video import videoMessageFactory as _

from Products.CMFCore.utils import getToolByName
from zope.component import getMultiAdapter
from Acquisition import aq_inner

def getImageUrl(resource):
    if ((IRTVideo.providedBy(resource) and \
                resource.hasSplashScreenImage()) or \
                    (not IRTVideo.providedBy(resource) and \
                                    resource.hasSplashScreenImage)):
        if not IRTVideo.providedBy(resource):
            return resource.getURL()+'/image_thumb'
        return resource.absolute_url()+'/image_thumb'
    portal = getToolByName(resource, 'portal_url').getPortalObject()
    return portal.absolute_url() +\
                "/++resource++collective.flowplayer.css/play.png"


class IRTVideoPortlet(IPortletDataProvider):
    """A portlet which can display video gallery"""

    header = schema.TextLine(title = _(u"label_portlet_header",
                                        default = u"Portlet header"),
                             description = _(u"help_portlet_header",
                                             default = u"Title of the rendered portlet"),
                             required = False)

    target = schema.Choice(title = _(u"label_target_object",
                                        default = u"Target object"),
                           description = _(u"help_target_object",
                                        default = u"This can be a file "\
                                            "containing an video content, "\
                                            "or a folder or collection "\
                                            "containing videos"),
                           required = True,
                           source = SearchableTextSourceBinder(
                                      {'object_provides': [IATTopic.__identifier__,
                                            IATFolder.__identifier__,
                                            IRTVideo.__identifier__]},
                                      default_query = 'path:'))

    limit = schema.Int(title = _(u"label_number_of_videos_to_show",
                            default = u"Number of videos to show"),
                       description = _(u"help_number_of_videos_to_show",
                            default = u"Enter a number greater than 0 "\
                                    "to limit the number of items displayed"),
                       required = False,
                       default = 0)

    show_more = schema.Bool(title = _(u"label_show_more_link",
                                default=u'Show "more..." link'),
                       description = _(u"help_show_more_link",
                                    default = u"If enabled, a more... "\
                                        "link will appear in the footer of "\
                                        "the portlet, "
                                    "linking to the underlying data."),
                       required = True,
                       default = True)


class Assignment(base.Assignment):
    implements(IRTVideoPortlet)

    header = u""
    target = None
    limit = 0
    show_more = False

    def __init__(self, header=u"", target=None, limit=None, show_more=False):
        self.header = header
        self.target = target
        self.limit = limit
        self.show_more = show_more

    @property
    def title(self):
        if self.header:
            return _(u"Video gallery")+': ' + self.header
        return _(u"Video gallery")


class Renderer(base.Renderer):
    render = ViewPageTemplateFile('portlet.pt')

    @property
    def available(self):
        return len(self.videos()) > 0

    def target_url(self):
        target=self.target()
        if target is None:
            return None
        plone_view = getMultiAdapter((aq_inner(target), self.request), name='plone')
        if plone_view.isStructuralFolder():
            return target.absolute_url()
        else:
            return "%s/view" %target.absolute_url()

    @memoize
    def videos(self):
        target = self.target()
        limit = self.data.limit
        if target is None:
            return []
        if IRTVideo.providedBy(target):
            return [dict(title=target.Title(),
                         description=target.Description(),
                         url=target.absolute_url(),
                         year=target.getYear(),
                         duration=target.getDuration(),
                         image_url=getImageUrl(target)),
                   ]
        if IATFolder.providedBy(target):
            values = []
            videos = target.getFolderContents(
                        contentFilter = {'object_provides':
                                            IRTVideo.__identifier__})
            for v in videos:
                values.append(dict(title = v.Title,
                                   description = v.Description,
                                   url = v.getURL(),
                                   year = v.getYear,
                                   duration = v.getDuration,
                                   image_url = getImageUrl(v),
                                   ))
            return (limit and values[:limit]) or values

        if IATTopic.providedBy(target):
            values = []
            videos = target.queryCatalog(
                    contentFilter={'object_provides':
                                    IRTVideo.__identifier__})
            for v in videos:
                values.append(dict(title=v.Title,
                                   description=v.Description,
                                   url=v.getURL(),
                                   year=v.getYear,
                                   duration=v.getDuration,
                                   image_url=getImageUrl(v),
                                   ))
            return (limit and values[:limit]) or values
        return []

    @memoize
    def target(self):
        target_path = self.data.target
        if not target_path:
            return None

        if target_path.startswith('/'):
            target_path = target_path[1:]

        if not target_path:
            return None

        portal_state = getMultiAdapter((self.context, self.request),
                                            name=u'plone_portal_state')
        portal = portal_state.portal()
        return portal.restrictedTraverse(target_path, default=None)


class AddForm(base.AddForm):
    form_fields = form.Fields(IRTVideoPortlet)
    form_fields['target'].custom_widget = UberSelectionWidget

    label = _(u"label_add_video_portlet",
                    default = u"Add Video Portlet")
    description = _(u"help_add_video_portlet",
                        default = u"This portlet display a video gallery.")

    def create(self, data):
        return Assignment(**data)


class EditForm(base.EditForm):
    form_fields = form.Fields(IRTVideoPortlet)
    form_fields['target'].custom_widget = UberSelectionWidget

    label = _(u"label_edit_video_portlet",
                default = u"Edit Video Portlet")
    description = _(u"help_edit_video_portlet",
                default = u"This portlet display a video gallery.")



# -*- coding: utf-8 -*-

import logging
from Products.CMFCore.utils import getToolByName

PROFILE_ID = 'profile-redturtle.video:default'

_PROPERTIES = [
    dict(name='default_videosize_w', type_='int', value=400),
    dict(name='default_videosize_h', type_='int', value=300),
]

# form : http://maurits.vanrees.org/weblog/archive/2009/12/catalog
def addKeyToCatalog(portal, logger=None):
    '''Takes portal_catalog and adds a key to it
    @param portal: context providing portal_catalog
    '''
    if logger is None:
        # Called as upgrade step: define our own logger.
        logger = logging.getLogger('redturtle.video')

    catalog = getToolByName(portal, 'portal_catalog')
    indexes = catalog.indexes()
    # Specify the indexes you want, with ('index_name', 'index_type')
    wanted = (('hasSplashScreenImage', 'FieldIndex'),
              ('getYear', 'KeywordIndex'),
              ('getDuration', 'KeywordIndex')
              )

    indexables = []
    for name, meta_type in wanted:
        if name not in indexes:
            catalog.addIndex(name, meta_type)
            indexables.append(name)
            logger.info("Added %s for field %s.", meta_type, name)
    if len(indexables) > 0:
        logger.info("Indexing new indexes %s.", ', '.join(indexables))
        catalog.manage_reindexIndex(ids=indexables)


def registerProperties(context, logger=None):
    ptool = getToolByName(context, 'portal_properties')
    props = ptool.redturtle_video_properties
    
    for prop in _PROPERTIES:
        if not props.hasProperty(prop['name']):
            props.manage_addProperty(prop['name'], prop['value'], prop['type_'])
            logger.info("Added missing %s property" % prop['name'])

def setupVarious(context):
    if context.readDataFile('redturtle.video_various.txt') is None:
        return

    logger = context.getLogger('redturtle.video')
    portal = context.getSite()
    addKeyToCatalog(portal, logger)
    registerProperties(portal, logger)


def migrateTo060rc1(context, logger=None):
    if logger is None:
        logger = logging.getLogger('redturtle.video')
    setup_tool = getToolByName(context, 'portal_setup')
    setup_tool.runImportStepFromProfile(PROFILE_ID, 'properties')
    registerProperties(context, logger)
    logger.info("Migrated to 0.6.0")




"""Definition of the RTRemoteVideo content type
"""

from zope.interface import implements

from AccessControl import ClassSecurityInfo
from Products.CMFCore import permissions
from Products.Archetypes import atapi
from Products.ATContentTypes.content import schemata
from Products.ATContentTypes.content.link import ATLink, ATLinkSchema
from Products.ATContentTypes.content.image import ATImageSchema
from Products.ATContentTypes.lib.imagetransform import ATCTImageTransform

from redturtle.video import videoMessageFactory as _
from redturtle.video.interfaces import IRTRemoteVideo
from redturtle.video.config import PROJECTNAME
from redturtle.video.content.default import DefaultVideo
from redturtle.video.content.video_schema import VIDEO_SCHEMA

RTRemoteVideoSchema = ATLinkSchema.copy() + VIDEO_SCHEMA.copy() + atapi.Schema((

    # -*- Your Archetypes field definitions here ... -*-

))

# Set storage on fields copied from ATContentTypeSchema, making sure
# they work well with the python bridge properties.

RTRemoteVideoSchema['title'].storage = atapi.AnnotationStorage()
RTRemoteVideoSchema['description'].storage = atapi.AnnotationStorage()

imageField = ATImageSchema['image'].copy()
imageField.required = False
imageField.primary = False
imageField.widget.description = _(u'help_video_image',
                                  default = u'Can be used to provide splash image when needed')
imageField.validators = None

RTRemoteVideoSchema.addField(imageField)
RTRemoteVideoSchema.moveField('image', after = 'remoteUrl')
RTRemoteVideoSchema['remoteUrl'].widget.size = 60

schemata.finalizeATCTSchema(RTRemoteVideoSchema, moveDiscussion = False)


class RTRemoteVideo(ATLink, ATCTImageTransform, DefaultVideo):
    """A link to a video with screenshot"""
    implements(IRTRemoteVideo)

    meta_type = "RTRemoteVideo"
    schema = RTRemoteVideoSchema

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')

    # -*- Your ATSchema to Python Property Bridges Here ... -*-

    security = ClassSecurityInfo()

    security.declareProtected(permissions.View, 'tag')
    def tag(self, **kwargs):
        """Generate image tag using the api of the ImageField
        """
        return self.getField('image').tag(self, **kwargs)

    def __bobo_traverse__(self, REQUEST, name):
        """Transparent access to image scales
        """
        if name.startswith('image'):
            field = self.getField('image')
            image = None
            if name == 'image':
                image = field.getScale(self)
            else:
                scalename = name[len('image_'):]
                scalename.replace(".jpg", "")
                if scalename in field.getAvailableSizes(self):
                    image = field.getScale(self, scale=scalename)
            if image is not None and not isinstance(image, basestring):
                # image might be None or '' for empty images
                return image

        return ATLink.__bobo_traverse__(self, REQUEST, name)

    def hasSplashScreenImage(self):
        """Boolean value to know if an image is available"""
        if self.getImage():
            return True
        return False

atapi.registerType(RTRemoteVideo, PROJECTNAME)



# -*- coding: utf-8 -*-

import tempfile
import urllib2
from urlparse import urlparse

from zope.interface import alsoProvides, noLongerProvides
from zope.component import getMultiAdapter, ComponentLookupError

from collective.flowplayer.interfaces import IFlowPlayable

from redturtle.video.metadataextractor import extract
from redturtle.video.interfaces import IVideoEmbedCode

def _setVideoMetadata(object, name):
    """Set the metadata taken from the video using hachoir
    """
    metadata = extract(name)
    if metadata is not None:
        # duration
        try:
            duration = metadata.getItems('duration')
            if len(duration) >= 1:
                strdate=str(duration[0].value)
                strdate=strdate.split('.')
                strdate=strdate[0]
                if strdate.startswith('0:'):
                    strdate = '0' + strdate
                object.setDuration(strdate)
        # no valid data
        except ValueError:
            pass
        except IndexError:
            pass

        # size
        try:
            width = metadata.getItems('width')[0].value
            height = metadata.getItems('height')[0].value
            object.setWidth(width)
            object.setHeight(height)
        # no valid data
        except ValueError:
            pass
        except IndexError:
            pass


def createTempFileInternalVideo(object, event):
    """Create a temporary file for InternalVideo
    """
    if object.getDuration():
        return
    file=object.getFile()
    fd=tempfile.NamedTemporaryFile()
    if type(file.data)==str:
        # *** blob support ***
        fd.write(file.data)
        #name = file.getBlob()._current_filename()
        name = fd.name
    else:
        # *** No blob ***
        fd.write(str(file.data))
        name = fd.name
    _setVideoMetadata(object, name)
    fd.close()
    object.reindexObject()


def createTempFileRemoteVideo(object, event):
    """Create a temporary file for RemoteVideo
    """
    if object.getDuration():
        return
    url=object.getRemoteUrl()
    response = urllib2.urlopen(url)
    fd=tempfile.NamedTemporaryFile()
    fd.write(response.read())
    _setVideoMetadata(object, fd.name)
    object.reindexObject()
    fd.close()


def externalVideoModified(object, event):
    """A remote video has been modified; check is we need to provide to it
    the IFlowPlayable interface (only if it is Flowplayer compatible)"""
    video_site = urlparse(object.getRemoteUrl())[1].replace('www.','')
    try:
        adapter = getMultiAdapter((object, object.REQUEST), IVideoEmbedCode, name = video_site)
        noLongerProvides(object, IFlowPlayable)
    except ComponentLookupError:
        adapter = getMultiAdapter((object, object.REQUEST), IVideoEmbedCode)            
        alsoProvides(object, IFlowPlayable)
    object.reindexObject(idxs=['object_provides'])

    


# -*- coding: utf-8 -*-

from zope.event import notify
from redturtle.video.tests.base import TestCase
from Products.Archetypes.event import ObjectInitializedEvent

from redturtle.video.migrator import migrateFlowplayerToRedTurtleVideo

class TestView(TestCase):

    def afterSetUp(self):
        self.setRoles(('Contributor', ))
        portal = self.portal
        portal.invokeFactory(type_name="File", id="one-atfile")
        portal.invokeFactory(type_name="Folder", id="folder")
        portal.folder.invokeFactory(type_name="File", id="another-atfile")
        f = getattr(portal, 'one-atfile')
        f.edit(title="A simple ATFile", file=self.getVideoFile())
        f.setFilename('1-2-3.mp4')
        notify(ObjectInitializedEvent(self.portal['one-atfile']))
        f = getattr(portal.folder, 'another-atfile')
        f.edit(title="Another simple ATFile", file=self.getVideoFile())
        f.setFilename('1-2-3.mp4')
        notify(ObjectInitializedEvent(self.portal.folder['another-atfile']))
        

    def test_siteMigration(self):
        output = migrateFlowplayerToRedTurtleVideo(self.portal)
        self.assertEqual(output,
                         ['Migrating /plone/one-atfile (File -> RTInternalVideo)',
                          'Migrating /plone/folder/another-atfile (File -> RTInternalVideo)'])


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestView))
    return suite



# -*- coding: utf-8 -*-

from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName

class DefaultVideo(object):
    """Common properties for rt video contents"""

    security = ClassSecurityInfo()
    
    security.declarePrivate("getDefaultWidth")
    def getDefaultWidth(self):
        p_tool = getToolByName(self, 'portal_properties')
        try:
            return p_tool.redturtle_video_properties.default_videosize_w
        except AttributeError:
            return 251 # old default

    security.declarePrivate("getDefaultHeight")
    def getDefaultHeight(self):
        p_tool = getToolByName(self, 'portal_properties')
        try:
            return p_tool.redturtle_video_properties.default_videosize_h
        except AttributeError:
            return 200 # old default



# -*- coding: utf-8 -*-

from urlparse import urlparse
from zope.component import getMultiAdapter, ComponentLookupError
from zope.interface import implements

from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile

from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName
from plone.memoize.instance import memoize

from collective.flowplayer.browser.view import File, Link
from collective.flowplayer.interfaces import IFlowPlayerView

from redturtle.video.interfaces import IVideoEmbedCode


class IRTInternalVideoView(IFlowPlayerView):
    """
    Interface for the internal RedTurtle video view
    """
    
    def getEmbedCode():
        """Get the code needed embed of the video in 3rd party pages""" 


class IRTExternalVideoView(IFlowPlayerView):
    """
    Interface for the remote RedTurtle video view
    """
    
    def getPlayerCode():
        """Get the code needed display the remoted video""" 


class Macros(BrowserView):
    template = ViewPageTemplateFile('redturtlevideo_macros.pt')

    def __getitem__(self, key):
        return self.template.macros[key]


class InternalVideo(File):
    """The Internal Video browser view"""
    
    implements(IRTInternalVideoView)

    def __init__(self, context, request):
        File.__init__(self, context, request)

        self.height = self.height or context.getHeight() or context.getDefaultHeight()
        self.width = self.width or context.getWidth() or context.getDefaultWidth()     
        self._scale = "height: %dpx; width: %dpx;" % (self.height, self.width)

    def href(self):
        return self.context.absolute_url()+'/at_download/file'

    def getEmbedCode(self):
        """Return embed code"""
        context = self.context
        portal_url = getToolByName(context, 'portal_url')()
        fpUrl = portal_url+"/%2B%2Bresource%2B%2Bcollective.flowplayer/flowplayer.swf"
        embed = """
        <object width="%(width)d" height="%(height)d" data="%(fpUrl)s" type="application/x-shockwave-flash">
                <param name="movie" value="%(fpUrl)s" />
                <param name="allowfullscreen" value="true" />
                <param name="allowscriptaccess" value="always" />
                <param name="flashvars" value="config=%(here_url)s/config.js" />
        </object>
    """ % {
           "fpUrl"   : fpUrl,
           "baseUrl" : portal_url,
           "here_url": context.absolute_url(),
           "clipUrl" : self.href(),
           "width"   : self.width,
           "height"   : self.height,           
           }
        return "".join((x.strip() for x in embed.splitlines()))


class RemoteVideo(Link):
    """The External Video link browser view"""
    
    implements(IRTExternalVideoView)

    @memoize
    def getPlayerCode(self):
        """Return code from external service player"""
        video_site = urlparse(self.context.getRemoteUrl())[1].replace('www.','')
        try:
            adapter = getMultiAdapter((self.context, self.request), IVideoEmbedCode, name = video_site)
        except ComponentLookupError:
            adapter = getMultiAdapter((self.context, self.request), IVideoEmbedCode)            
        return adapter()


class InternalVideoConfiguration(InternalVideo):
    """Return the right configuration js file for this video"""
    
    MODEL = """{'clip':{'scaling':'fit',
                                                       'autoBuffering':false,
                                                       'autoPlay':false,
                                                       'baseUrl':'%(baseUrl)s',
                                                       'url':'%(clipUrl)s'
                                        },
                               'canvas':{'backgroundColor':'#112233'},
                               'plugins':{'controls':{'time':false,
                                                      'volume':false,
                                                      'fullscreen':false},
                                          'content':{'url':'flowplayer.swf',
                                                     'html':'Flash plugins work too'}
                                         }
                                }"""

    def __call__(self):
        portal_url = getToolByName(self.context, 'portal_url')()
        fpUrl = portal_url+"/%2B%2Bresource%2B%2Bcollective.flowplayer/flowplayer.swf"
        self.request.response.setHeader("Content-type", "application/x-javascript")

        return self.MODEL % {
           "fpUrl"   : fpUrl,
           "baseUrl" : portal_url,
           "clipUrl" : self.href()
           }



# -*- coding: utf-8 -*-

from Products.ATContentTypes.interface.link import IATLink
from redturtle.video.interfaces import IRTVideo

class IRTRemoteVideo(IATLink, IRTVideo):
    """A link to a video with screenshot"""




from Products.Archetypes import atapi
from redturtle.video import videoMessageFactory as _

VIDEO_SCHEMA=atapi.Schema((

    atapi.StringField('year',
                widget = atapi.StringWidget(
                    label = _(u'label_year', default=u'Year'),
                    description = '',
                    )),

    atapi.StringField('duration',
                widget = atapi.StringWidget(
                    label = _(u'label_duration', default=u'Duration'),
                    description = '',
                    )),
                    
    atapi.BooleanField('useSplashScreen',
                default=False,
                schemata="look",
                widget = atapi.BooleanWidget(
                    label = _(u'label_use_imge', default=u'Use image as splash screen?'),
                    description = _(u'help_use_imge',
                                    default=u"Check for use splash screen image not only in the product's portlet "
                                            u"but also in the video view."),
                    )),

    atapi.IntegerField('width',
                validation=('isInt',),
                default_method="getDefaultWidth",
                schemata="look",
                widget = atapi.IntegerWidget(
                    label = _(u'label_width', default=u'Video width'),
                    description = '',
                    )),

    atapi.IntegerField('height',
                validation=('isInt',),
                default_method="getDefaultHeight",
                schemata="look",
                widget = atapi.IntegerWidget(
                    label = _(u'label_height', default=u'Video height'),
                    description = '',
                    )),

))



# -*- coding: utf-8 -*-

from zope.interface import Interface
from zope.formlib import form
try: # >= 4.1
    from five.formlib import formbase
except ImportError: # < 4.1
    from Products.Five.formlib import formbase
from redturtle.video.migrator import migrateFlowplayerToRedTurtleVideo
from Products.statusmessages.interfaces import IStatusMessage

from redturtle.video import videoMessageFactory as _


class IMigrateFlowplayerSchema(Interface):
    pass


class MigrateFlowplayerFile(formbase.PageForm):
    form_fields = form.FormFields(IMigrateFlowplayerSchema)
    label = u'Video files migration'
    description = u'Migrate basic Flowplayer video files to RedTurtle Video'

    @form.action(_(u'Migrate Flowplayer video files'))
    def actionMigrate(self, action, data):
        output = migrateFlowplayerToRedTurtleVideo(self.context)
        if output:
            for o in output:
                IStatusMessage(self.request).addStatusMessage(o, type='info')
        else:
            IStatusMessage(self.request).addStatusMessage(_(u'Nothing to migrate'), type='info')
        return self.request.response.redirect(self.context.absolute_url())

    @form.action(_(u'Cancel'))
    def actionCancel(self, action, data):
        return self.request.response.redirect(self.context.absolute_url())



# -*- coding: utf-8 -*-

import unittest
from zope.testing import doctest
import redturtle.video.browser.videoembedcode

import zope.component
import redturtle.video.interfaces

import zope.interface
from zope.traversing.adapters import DefaultTraversable
from zope.traversing.interfaces import ITraversable


def setUp(test):

    zope.component.provideAdapter(
            DefaultTraversable,
            (zope.interface.Interface,),
            provides=ITraversable
        )
    
    zope.component.provideAdapter(
            redturtle.video.browser.videoembedcode.VideoEmbedCode,
            (redturtle.video.interfaces.IRTRemoteVideo,
             zope.publisher.interfaces.browser.IHTTPRequest),
            provides=redturtle.video.interfaces.IVideoEmbedCode
        )

def test_suite():
    return unittest.TestSuite((
        doctest.DocTestSuite(redturtle.video.browser.videoembedcode,
                     setUp=setUp,
                     optionflags=doctest.REPORT_ONLY_FIRST_FAILURE | doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS,),
        ))



# -*- coding: utf-8 -*-

from zope.component import adapts
from zope.interface import implements

from archetypes.schemaextender.interfaces import ISchemaExtender
from archetypes.schemaextender.field import ExtensionField
from plone.app.blob.field import BlobField

from Products.Archetypes import atapi

from redturtle.video.interfaces import IRTInternalVideo
from redturtle.video.content.rtinternalvideo import RTInternalVideoSchema

class ExtensionBlobField(ExtensionField, BlobField):
    """ derivative of blobfield for extending schemas """


class RemoteVideoATTypeExtender(object):
    adapts(IRTInternalVideo)
    implements(ISchemaExtender)

    fields = [
        ExtensionBlobField('file',
            widget=atapi.FileWidget(
                label=RTInternalVideoSchema['file'].widget.label,
                description=RTInternalVideoSchema['file'].widget.description,
            ),
            required=True,
            primary=True,
            validators=('isNonEmptyFile'),
        ),

    ]

    def __init__(self, context):
        self.context = context

    def getFields(self):
        return self.fields



# -*- coding: utf-8 -*-

import os, sys
from setuptools import setup, find_packages

version = '0.7.1'

tests_require=['zope.testing', 'Products.contentmigration']

install_requires = ['setuptools',
                    'hachoir_core',
                    'hachoir_metadata',
                    'hachoir_parser',
                    'collective.flowplayer',
                    'plone.app.imaging>=1.0b9',
                    ]

# what I read there seems not working propery for Plone 3.3
# http://plone.org/documentation/manual/upgrade-guide/version/upgrading-plone-4.0-to-4.1/referencemanual-all-pages
if sys.version_info < (2, 6):
    install_requires.append('Plone')
    install_requires.append('collective.relateditems')
else:
    install_requires.append('Products.CMFPlone')

setup(name='redturtle.video',
      version=version,
      description="Basic video contents (site's internal and remote) for Plone; "
                  "use collective.flowplayer but also pluggable with external services",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 3.3',
        'Framework :: Plone :: 4.0',
        'Framework :: Plone :: 4.1',
        'Development Status :: 5 - Production/Stable',
        'Topic :: Multimedia :: Video',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        ],
      keywords='plone video flowplayer plonegov',
      author='RedTurtle Technology',
      author_email='sviluppoplone@redturtle.net',
      url='http://plone.org/products/redturtle.video',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['redturtle', ],
      include_package_data=True,
      zip_safe=False,
      install_requires=install_requires,
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      test_suite = 'redturtle.video.tests.test_doctest.test_suite',
      entry_points="""
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



# -*- coding: utf-8 -*-

from zope.interface import Interface
from zope.formlib import form
try: # >= 4.1
    from five.formlib import formbase
except ImportError: # < 4.1
    from Products.Five.formlib import formbase
from redturtle.video.migrator import migrateRTInternalVideo
from Products.statusmessages.interfaces import IStatusMessage

from redturtle.video import videoMessageFactory as _

try:
    import Products.contentmigration
    MIGRATION_MODULE = True
except ImportError:
    MIGRATION_MODULE = False

class IMigrateBlobsSchema(Interface):
    pass


class MigrateBlobs(formbase.PageForm):
    form_fields = form.FormFields(IMigrateBlobsSchema)
    label = u'Blobs Migration'
    description = u'Migrate video, making it use plone.app.blob'

    @form.action(_(u'Migrate Internal video'))
    def actionMigrate(self, action, data):
        if MIGRATION_MODULE:
            output = migrateRTInternalVideo(self.context)
            IStatusMessage(self.request).addStatusMessage(output, type='info')
            return self.request.response.redirect(self.context.absolute_url())
        else:
            output = _(u'migration_error_msg',
                       default=u'You need to install "Products.contentmigration" product for perform the task')
            IStatusMessage(self.request).addStatusMessage(output, type='error')
            return self.request.response.redirect(self.context.absolute_url()+'/@@migrateblobs')
        

    @form.action(_(u'Cancel'))
    def actionCancel(self, action, data):
        return self.request.response.redirect(self.context.absolute_url())
